#' Check the validity of a \code{compData} object
#' 
#' Check the validity of a \code{compData} object. An object that passes the check can be used as the input for the differential expression analysis methods interfaced by \code{compcodeR}.
#' 
#' @export 
#' @author Charlotte Soneson
#' @param object A \code{compData} object
#' @examples
#' \dontrun{
#' mydata <- generateSyntheticData(dataset = "mydata", n.vars = 12500, 
#'                                 samples.per.cond = 5, n.diffexp = 1250)
#' check_compData(mydata)
#' }
check_compData <- function(object) {
  if (!isS4(object)) {
    return("This is not an S4 object.")
  } else {
    if (length(object@count.matrix) == 0) {
      return("Object must contain a non-empty count matrix.")
    }
    
    if (length(object@sample.annotations) == 0) {
      return("Object must contain a non-empty sample annotation data frame.")
    }
    
    if (length(object@sample.annotations$condition) == 0) {
      return("The sample.annotations must contain a column named condition.")
    }
    
    if (length(object@info.parameters) == 0) {
      return("Object must contain a non-empty list called info.parameters.")
    }
    
    if (length(object@info.parameters$dataset) == 0) {
      return("info.parameters list must contain an entry named 'dataset'.")
    }
    
    if (length(object@info.parameters$uID) == 0) {
      return("info.parameters list must contain an entry named 'uID'.")
    }
    
    if (length(object@count.matrix) != 0 && 
          length(object@variable.annotations) != 0) {
      if (nrow(object@count.matrix) != 
            nrow(object@variable.annotations)) {
        return("count.matrix and variable.annotations do not contain the same number of rows.")
      } else {
        if (length(rownames(object@count.matrix)) != 0 && 
              length(rownames(object@variable.annotations)) != 0 && 
              rownames(object@count.matrix) != 
              rownames(object@variable.annotations)) {
          return("The rownames of count.matrix and variable.annotations are not the same.")
        }
      } 
    } 
    
    if (length(object@count.matrix) != 0 && 
          length(object@sample.annotations) != 0) {
      if (ncol(object@count.matrix) != nrow(object@sample.annotations)) {
        return("The number of columns of count.matrix is different from the number of rows of sample.annotations.")
      } else {
        if (length(colnames(object@count.matrix)) != 0 && 
              length(rownames(object@sample.annotations)) != 0 && 
              colnames(object@count.matrix) != rownames(object@sample.annotations)) {
          return("The colnames of count.matrix are different from the rownames of sample.annotations.")
        }
      } 
    }
  }
    
  return(TRUE)
}

#' Check the validity of a \code{compData} result object
#' 
#' Check the validity of a \code{compData} object with differential expression results. An object that passes the check can be used as the input for the method comparison functions in \code{compcodeR}.
#' 
#' @export 
#' @author Charlotte Soneson
#' @param object A \code{compData} object
#' @examples
#' \dontrun{
#' mydata <- generateSyntheticData(dataset = "mydata", n.vars = 12500, 
#'                                 samples.per.cond = 5, n.diffexp = 1250,
#'                                 output.file = "mydata.rds")
#' ## Check an object without differential expression results
#' check_compData_results(mydata)
#' 
#' runDiffExp(data.file = "mydata.rds", result.extent = "voom.limma", 
#'            Rmdfunction = "voom.limma.createRmd", 
#'            output.directory = ".", norm.method = "TMM")
#' resdata <- readRDS("mydata_voom.limma.rds")
#' ## Check an object containing differential expression results
#' check_compData_results(resdata)
#' }
check_compData_results = function(object) {
  tmp <- check_compData(object)
  if (is.character(tmp)) {
    return(tmp)
  } else {
    if(check_compData(object)) {
      if (length(object@method.names) == 0) {
        return("Object must contain a list named 'method.names' identifying the differential expression method used.")
      }
      
      if (length(object@result.table) == 0) {
        return("Object must contain a data frame named 'result.table'.")
      }
      
      if (length(object@result.table$score) == 0) {
        return("result.table must contain a column named 'score'.")
      }
    }
  }
  return(TRUE)
}

#' Class compData
#'
#' Class \code{compData} defines a count data set, including the count matrix, sample and variable annotations and various information parameters regarding the simulation details. It can also contain results from a differential expression analysis. 
#'
#' @name compData-class
#' @rdname compData-class
#' @exportClass compData
setClass(
  Class = "compData", 
  
  representation = representation(count.matrix = "matrix",
                                  sample.annotations = "data.frame",
                                  info.parameters = "list",
                                  filtering = "character",
                                  variable.annotations = "data.frame",
                                  analysis.date = "character",
                                  package.version = "character",
                                  method.names = "list",
                                  code = "character",
                                  result.table = "data.frame"),
  
  validity = check_compData)

#' Create a \code{compData} object
#' 
#' Create a \code{compData} object. The object contains information about a data set, such as the count matrix and sample and variable annotation. It can also contain results from a differential expression analysis.
#' 
#' @param count.matrix A count matrix, with genes as rows and observations as columns
#' @param sample.annotations A data frame, containing at least one column named 'condition', encoding the grouping of the observations into two groups. The row names should be the same as the column names of the \code{count.matrix}.
#' @param info.parameters A list containing information regarding simulation parameters etc. The only mandatory entries are \code{dataset} and \code{uID}, but it may contain entries such as the ones listed below (see \code{generateSyntheticData} for more detailed information about each of these entries).
#' \itemize{
#' \item \code{dataset}: a name or identifier of the data set (e.g., summarizing the simulation settings).
#' \item \code{samples.per.cond}
#' \item \code{n.diffexp} 
#' \item \code{repl.id}
#' \item \code{seqdepth}
#' \item \code{minfact}
#' \item \code{maxfact}
#' \item \code{fraction.upregulated}
#' \item \code{between.group.diffdisp}
#' \item \code{filter.threshold.total}
#' \item \code{filter.threshold.mediancpm}
#' \item \code{fraction.non.overdispersed}
#' \item \code{random.outlier.high.prob}
#' \item \code{random.outlier.low.prob}
#' \item \code{single.outlier.high.prob}
#' \item \code{single.outlier.low.prob}
#' \item \code{effect.size}
#' \item \code{uID}: a unique ID that is given to each data set.
#' }
#' @param variable.annotations A data frame with variable annotations (number of rows equal to the number of variables in the data set). Not mandatory, but may contain columns such as the ones listed below. If present, the row names should be the same as the row names of the \code{count.matrix}.
#' \itemize{
#' \item \code{truedispersions.S1}: the true dispersion for each gene in condition S1.
#' \item \code{truedispersions.S2}: the true dispersion for each gene in condition S2.
#' \item \code{truemeans.S1}: the true mean value for each gene in condition S1.
#' \item \code{truemeans.S2}: the true mean value for each gene in condition S2.
#' \item \code{n.random.outliers.up.S1}: the number of 'random' outliers with extremely high counts for each gene in condition S1.
#' \item \code{n.random.outliers.up.S2}: the number of 'random' outliers with extremely high counts for each gene in condition S2.
#' \item \code{n.random.outliers.down.S1}: the number of 'random' outliers with extremely low counts for each gene in condition S1.
#' \item \code{n.random.outliers.down.S2}: the number of 'random' outliers with extremely low counts for each gene in condition S2.
#' \item \code{n.single.outliers.up.S1}: the number of 'single' outliers with extremely high counts for each gene in condition S1.
#' \item \code{n.single.outliers.up.S2}: the number of 'single' outliers with extremely high counts for each gene in condition S2.
#' \item \code{n.single.outliers.down.S1}: the number of 'single' outliers with extremely low counts for each gene in condition S1.
#' \item \code{n.single.outliers.down.S2}: the number of 'single' outliers with extremely low counts for each gene in condition S2.
#' \item \code{M.value}: the M-value (observed log2 fold change between condition S1 and condition S2) for each gene.
#' \item \code{A.value}: the A-value (observed average expression level across condition S1 and condition S2) for each gene.
#' \item \code{truelog2foldchanges}: the true (simulated) log2 fold changes between condition S1 and condition S2.
#' \item \code{upregulation}: a binary vector indicating which genes are simulated to be upregulated in condition S2 compared to condition S1.
#' \item \code{downregulation}: a binary vector indicating which genes are simulated to be downregulated in condition S2 compared to condition S1.
#' \item \code{differential.expression}: a binary vector indicating which genes are simulated to be differentially expressed in condition S2 compared to condition S1.
#' }
#' @param filtering A character string containing information about the filtering that has been applied.
#' @param analysis.date If a differential expression analysis has been performed, a character string detailing on which day it was run.
#' @param package.version If a differential expression analysis has been performed, a character string giving the version of the differential expression packages that were applied. 
#' @param method.names If a differential expression analysis has been performed, a list with entries \code{full.name} and \code{short.name}, giving the full name of the differential expression method (may including version number and parameter settings) and a short name. 
#' @param code If a differential expression analysis has been performed, a character string containing the code that was run to perform the analysis. This can be written to an HTML file using the \code{\link{generateCodeHTMLs}} function.
#' @param result.table If a differential expression analysis has been performed, a data frame containing the results of the analysis. The only mandatory column is \code{score}, which gives a score for each gene, where a higher score suggests a "more highly differentially expressed" gene. Different comparison functions use different columns of this table, if available. The list below gives the columns that are used by the implemented methods.
#' \itemize{
#' \item \code{pvalue} nominal p-values
#' \item \code{adjpvalue} p-values adjusted for multiple comparisons
#' \item \code{logFC} estimated log-fold changes between the two conditions
#' \item \code{score} the score that will be used to rank the genes in order of significance. Note that high scores always signify differential expression, that is, a strong association with the predictor. For example, for methods returning a nominal p-value the score can be defined as 1 - pvalue.
#' \item \code{FDR} false discovery rate estimates
#' \item \code{posterior.DE} posterior probabilities of differential expression
#' \item \code{prob.DE} conditional probabilities of differential expression
#' \item \code{lfdr} local false discovery rates
#' \item \code{statistic}  test statistics from the differential expression analysis
#' \item \code{dispersion.S1} dispersion estimates in condition 1
#' \item \code{dispersion.S2} dispersion estimates in condition 2
#' }
#' 
#' @export
#' @author Charlotte Soneson
#' @return A \code{compData} object containing the provided information.
#' @examples
#' count.matrix <- round(matrix(1000*runif(4000), 1000))
#' sample.annotations <- data.frame(condition = c(1, 1, 2, 2))
#' info.parameters <- list(dataset = "mydata", uID = "123456")
#' cpd <- compData(count.matrix, sample.annotations, info.parameters)
compData <- function(count.matrix, sample.annotations, 
                     info.parameters, variable.annotations = data.frame(), 
                     filtering = "no info", analysis.date = "", 
                     package.version = "", method.names = list(), 
                     code = "", result.table = data.frame()) {
  
  count.matrix <- as.matrix(count.matrix)
  mode(count.matrix) <- "numeric"
  
  sample.annotations <- data.frame(sample.annotations, 
                                   stringsAsFactors = FALSE)
  
  cmpd <- new("compData", count.matrix = count.matrix, 
              sample.annotations = sample.annotations, 
              info.parameters = info.parameters, 
              variable.annotations = variable.annotations, 
              filtering = filtering, 
              analysis.date = analysis.date, 
              package.version = package.version, 
              method.names = method.names,
              code = code, 
              result.table = result.table)
  
  validObject(cmpd)
  
  cmpd
}

#' Convert a list with data and results to a \code{compData} object
#' 
#' Given a list with data and results (resulting e.g. from \code{compcodeR} version 0.1.0), convert it to a \code{compData} object.
#' 
#' @export
#' @param inp.list A list with data and results, e.g. generated by \code{compcodeR} version 0.1.0.
#' @author Charlotte Soneson
#' @examples
#' convertListTocompData(list(count.matrix = matrix(round(1000*runif(4000)), 1000),
#'                            sample.annotations = data.frame(condition = c(1,1,2,2)),
#'                            info.parameters = list(dataset = "mydata", 
#'                            uID = "123456")))
convertListTocompData <- function(inp.list) {
  if (is.null(inp.list$count.matrix) || is.null(inp.list$sample.annotations) || 
        is.null(inp.list$info.parameters) || !is.matrix(inp.list$count.matrix) || 
        !is.data.frame(inp.list$sample.annotations) || !is.list(inp.list$info.parameters)) {
    message("Cannot convert list to compData object (not all required slots are available).")
    return(NULL)
  } else {
    compData(count.matrix = inp.list$count.matrix, 
             sample.annotations = inp.list$sample.annotations, 
             info.parameters = inp.list$info.parameters, 
             variable.annotations = as.data.frame(inp.list$variable.annotations,
                                                  stringsAsFactors = FALSE), 
             filtering = as.character(inp.list$filtering), 
             analysis.date = as.character(inp.list$analysis.date), 
             package.version = as.character(inp.list$package.version), 
             method.names = as.list(inp.list$method.names), 
             code = as.character(inp.list$code), 
             result.table = as.data.frame(inp.list$result.table,
                                          stringsAsFactors = FALSE))
  }
}

#' Convert a \code{compData} object to a list
#' 
#' Given a \code{compData} object, convert it to a list.
#' 
#' @export
#' @param cpd A \code{compData} object
#' @author Charlotte Soneson
#' @examples
#' mydata.obj <- generateSyntheticData(dataset = "mydata", n.vars = 12500,
#'                                     samples.per.cond = 5, n.diffexp = 1250, 
#'                                     output.file = "mydata.rds")
#' mydata.list <- convertcompDataToList(mydata.obj)
convertcompDataToList <- function(cpd) {
  list(count.matrix = cpd@count.matrix,
       sample.annotations = cpd@sample.annotations,
       info.parameters = cpd@info.parameters,
       variable.annotations = cpd@variable.annotations,
       filtering = cpd@filtering,
       analysis.data = cpd@analysis.date,
       package.version = cpd@package.version,
       method.names = cpd@method.names,
       code = cpd@code,
       result.table = cpd@result.table)
}